/*
 * bf_brute2
 * by Javantea aka Joel R. Voss
 * Sept 23, 2006
 * Attack weak passwords on symmetric ciphers.
 */

#include <string.h>
#include "check_tar.h"
#include "increment.h"
#include <ssl_bf2a.h>

int decrypt_bfd(const BYTE *enc_data, int enc_size, const char *password, int pass_len, BYTE *out, int *data_size)
{
	EVP_CIPHER_CTX ctx;
	BYTE           key[KEY_LENGTH]; 
	BYTE           iv[EVP_MAX_IV_LENGTH];
	BYTE           salt[PKCS5_SALT_LEN];

	char *dec_data;
	const char *ptr_enc;
	char  final[EVP_MAX_BLOCK_LENGTH];
	int   ret = 0;

    // Take the encrypted data from the stdin.
	if(0 != strncmp(enc_data, BF_HEADER, 8))
	{
		fprintf(stderr, "Error: BF Decrypt Header not found: %8.8s.\n", enc_data);
		return 1;
	}
	memcpy(salt, enc_data + 8, PKCS5_SALT_LEN);
    
	generate_bin_key(key, password, pass_len, salt, iv, KEY_LENGTH);
    //print_hex(key, KEY_LENGTH);
	ptr_enc = enc_data + 8 + PKCS5_SALT_LEN;
    // Subtract the header from the encrypted size.
	enc_size -= (8 + PKCS5_SALT_LEN);
	
	EVP_CIPHER_CTX_init(&ctx);
	EVP_DecryptInit(&ctx, EVP_bf_cbc(), key, iv);
	int j, k = 0;
	for(j=0; j<enc_size; j+=8)
	{
		dec_data = decrypt_data(&ctx, ptr_enc + j, 8);
        // The first one is always null, ignore it or fix it.
        // TODO: You should probably try to understand it as well.
				   if(dec_data != 0)
		   {
			   memcpy(out + k, dec_data, 8);
			   free(dec_data);
            // Counter for output.
			   k += 8;
		   }
        //else
        //    fprintf(stderr, "Error: Decrypt failed, null data.\n");
	}
	if (!EVP_DecryptFinal(&ctx, final, data_size))
	{
		fprintf(stderr, "Error: Decrypt Final, padding incorrect.\n");
		ret = 1;
        //abort();
	}
	EVP_CIPHER_CTX_cleanup(&ctx);
	memcpy(out + k, final, *data_size);
    // Data size is set in DecryptFinal to the last size (0-7).
    // Add k (output counter) to get the correct total size.
	*data_size += k;

	return ret;
}

int bf_brute(const char *file_data, int size, const char *password)
{
	// 1) Decrypt file_data with password.
	char out_data[size];
	int out_size = -1;
	int ret = decrypt_bfd(file_data, size, password, strlen(password), out_data, &out_size);
	// 2) check_tar(out_data, out_size)
	return check_tar(out_data, out_size);
}

#ifdef __BF_BRUTE2_SA__

void usage()
{
	printf("Usage: bf_brute2 file.tar.bf [mypass000]\n");
}

int main(int argc, char **argv)
{
	if(argc < 2 || (strncmp(argv[1], "-h", 2) == 0))
	{
		usage();
		return 1;
	}
	// Attack a file to find the password.
	char indata[256];
	FILE *infile = fopen(argv[1], "rb");
	if(infile == 0)
	{
		printf("File is invalid: %s\n", argv[1]);
		usage();
		return 1;
	}
	int size = fread(indata, 1, 256, infile);
	fclose(infile);
	char *password = "0000";
	if(argc >= 3)
	{
		password = argv[2];
	}
	while(1)
	{
		if(bf_brute(indata, size, password) == 0)
		{
			printf("Possible password found: %s\n", password);
			return 0;
		}
		increment(password, strlen(password), password, '0', '~', 0);
	}
	return 1;
}
#endif // __BF_BRUTE2_SA__
